import React, { useState, useEffect } from 'react';
import { debounce } from 'lodash';
import { Checkbox } from 'antd';
import classnames from 'classnames';
import useMergeValue from 'use-merge-value';
import styles from './itemCard.less';

const CheckGroup = Checkbox.Group;

const getId = item => item.userID || item.roleID || item.stationID || item.id;

const checkIsGroup = node => ['widgetGroup'].includes(node?.type || node);

const checkChildrenByCondition = (item, fn, tag = 'every') => {
  if (!checkIsGroup(item)) {
    return fn(item);
  }
  return item.children[tag](t => checkChildrenByCondition(t, fn, tag));
};

const ListCardItem = props => {
  const {
    getValueCallback,
    itemid,
    searchWord,
    allChecked,
    groupIndeterminate,
    children,
    text,
    type,
    isChecked,
    ...rest
  } = props;

  // id
  const id = getId(rest);
  // 当前checkbox为分组时有用，是否部分选中
  const [indeterminate, setIndeterminate] = useState(true);
  // 用来收集子节点的值
  const [childValues, setChildValues] = useState({});

  // 是否是地图组件
  const isMapWedgt = type === 'widgetUIPage';

  // 当前组件是否是分组id
  const isGroup = checkIsGroup(type);
  const [groupChecked, setGroupChecked] = useState(false);

  // 当前组件是否勾选
  const [checked, setChecked] = useState(isChecked);

  useEffect(() => {
    // const all = children.every(u => checkAllChildrenChecked(u));
    if (isGroup) {
      // setIndeterminate(!all && children.some(u => checkSomeChildrenChecked(u)));
    } else {
      setIndeterminate(false);
      // 初始进来报告自己的值
      getValueCallback(isChecked ? [id] : [], itemid);
    }
  }, []);

  // 监听父级是否勾选，如果父级勾选 则需要吧自己设置勾选状态并回传值
  useEffect(() => {
    if (allChecked !== checked && !groupIndeterminate) {
      if (!isGroup) {
        setChecked(allChecked);
        setIndeterminate(false);
        getValueCallback(allChecked ? [id] : [], itemid);
      }
    }
  }, [allChecked, groupIndeterminate]);

  // 勾选事件处理
  const handleChecked = e => {
    const { checked: v } = e.target;
    if (isGroup) {
      if (indeterminate) {
        setGroupChecked(true);
      } else {
        setGroupChecked(v);
      }
      setIndeterminate(false);
    } else {
      setChecked(v);
    }
    if (!isGroup) {
      getValueCallback(v ? [id] : [], itemid);
    }
  };

  const handleChildValueCallback = (arr, childIndex) => {
    // 进这里的都是分组，组长统计并处理自己的状态： 全选，部分选，零选
    // 拿到所有勾选的子节点
    childValues[childIndex] = arr.flat();
    const childValArr = Object.values(childValues).flat();
    // 保存组员状态
    setChildValues({ ...childValues });

    // 递归执行的条件函数
    const condition = c => childValArr.includes(getId(c));
    const isAllchecked = checkChildrenByCondition(
      { children, type },
      condition,
    );
    const isSomeChecked = checkChildrenByCondition(
      { children, type },
      condition,
      'some',
    );
    setIndeterminate(!isAllchecked && isSomeChecked);
    setGroupChecked(isSomeChecked);
    // eslint-disable-next-line no-unused-expressions
    getValueCallback && getValueCallback([...childValArr, id], itemid);
  };

  const renderChild = () =>
    children &&
    children.map((c, i) => (
      <ListCardItem
        {...c}
        itemid={i}
        allChecked={groupChecked}
        groupIndeterminate={indeterminate}
        key={`item${i}key`}
        getValueCallback={handleChildValueCallback}
        searchWord={searchWord}
      />
    ));

  return (
    <>
      <div
        className={classnames({
          [styles.divBox]: isGroup,
          [styles.divSingle]: !isGroup,
        })}
      >
        <div className={styles.topCheckbox}>
          <Checkbox
            indeterminate={isGroup ? indeterminate : false}
            checked={isGroup ? groupChecked : checked}
            onChange={handleChecked}
          >
            {isMapWedgt ? (
              <div
                style={{ display: 'inline-block' }}
                ref={r => {
                  if (r) r.innerHTML = text;
                }}
              />
            ) : (
              <span
                className={classnames({
                  [styles.boldLabel]: isGroup,
                  [styles.isSearch]: searchWord && text.includes(searchWord),
                })}
              >
                {text}
              </span>
            )}
          </Checkbox>
        </div>
        <div style={{ width: '100%' }} className={styles.checkdiv}>
          {renderChild()}
        </div>
      </div>
    </>
  );
};

export default ListCardItem;
